/*
 * Atari Gamer's ROM Menu for BennVenn's ElCheapoSD Cartridge
 * Copyright 2022 Igor Kromin (atarigamer.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

#include <tgi.h>
#include "AgCommon.h"


/*
Following files are useful to reference:
- SUZY defs          https://github.com/cc65/cc65/blob/master/include/_suzy.h
- MIKEY defs         https://github.com/cc65/cc65/blob/master/include/_mikey.h
- CC65 function ref  https://cc65.github.io/doc/funcref.html#ss3.253
*/


// ----------------------------------------------------------------------------
// DEFINES
// ----------------------------------------------------------------------------

#define DELAY_LOGO_TIME 50
#define FONT_COLOUR_LOGO 0x0F
#define FONT_COLOUR_MENU 0x04
#define FONT_COLOUR_STORY 0x0F


// ----------------------------------------------------------------------------
// READ-ONLY DATA
// ----------------------------------------------------------------------------

// Default palette. This palette is loaded when the GFX system is initialised.
// https://atarigamer.com/pages/atari-lynx-palette-generator
#pragma bss-name (push, "RODATA")
u16 myPalette[] = {
	0x0000, // black       - index 0
  0x000f, // red         - index 1
  0x0f00, // greeen      - index 2
  0x00f0, // blue        - index 3
	0x0f0f, // yellow      - index 4
  0x00ff, // magenta     - index 5
  0x0ff0, // cyan        - index 6
  0x0009, // dark red    - index 7
  0x0900, // dark green  - index 8
  0x0090, // dark blue   - index 9
  0x0909, // dark yellow - index 10 (A)
  0x0099, // purple      - index 11 (B)
	0x0990, // aqua        - index 12 (C)
  0x0333, // dark grey   - index 13 (D)
  0x0999, // light grey  - index 14 (E)
  0x0fff  // white       - index 15 (F)
};
#pragma bss-name (pop)


// ----------------------------------------------------------------------------
// Local variables
// ----------------------------------------------------------------------------

static clock_t startTime;
static u8 playerX;

extern char img_player[];
static SCB_REHV_PAL sprPlayer = {
	BPP_4 | TYPE_NORMAL,                      // sprctl0
  REHV | PACKED,                            // sprctl1
  0,                                        // sprcoll
  0,                                        // next SCB PTR
  img_player,                               // SPR data PTR
  0, 0,                                     // offset (hpos, vpos)
  0x0100, 0x0100,                           // size (hsize, vsize)
  {0x01,0x23,0x45,0x67,0x89,0xAB,0xCD,0xEF} // penpal
};

// ----------------------------------------------------------------------------
// FORWARD FUNCTION DEFINITIONS
// ----------------------------------------------------------------------------

static void __fastcall__ logoScreen();
static void __fastcall__ gameMenuScreen();
static void __fastcall__ storyScreen();
static void __fastcall__ resetGame();
static void __fastcall__ updateAndDrawGame();


// ----------------------------------------------------------------------------
// CODE
// ----------------------------------------------------------------------------

void main(void) {
  agInitGfx();       // initialise graphics
  agSetPalette(myPalette);
  agInitRand8(42);   // initialise RNG (use your own seed other than 42)

  startTime = clock2();

  /*
  A number of 'screens' follow. Each one is responsible for updating graphics
  on screen, handling input and processing logic. Once a 'screen' function is
  done, the next screen function takes over.
  */

  logoScreen();      // startup logo screen
  gameMenuScreen();  // game menu screen, before you begin a game
  storyScreen();     // game story/instructions screen before gameplay starts
  
  resetGame();       // resets the game state to a clean state

  /*
  Main game logic, input processing, graphics updates, etc, are done in an
  infinite loop. This should handle drawing, input and processing for a single
  frame.
  */
  while(1) {
    updateAndDrawGame();
  }
}

// ----------------------------------------------------------------------------

static void __fastcall__ logoScreen() {
  AG_WAIT_LCD();

  /*
  set background image palette to show 'magenta' i.e. the 6th (index 5) colour
  from the palette above in this 1-bit image; then draw the sprite it's linked to
  */
  agSprBackground.penpal[0] = 0x05;
  tgi_sprite(&agSprBackground);

  // draw some text after setting it's colour
  agSetFontColor(FONT_COLOUR_LOGO);
  agDrawTextCenter(AG_LCD_MID_Y, "MY LOGO");
  
  // update the LCD
  tgi_updatedisplay();

  // wait some time before continuing to the next screen
  while (clock2() - startTime < DELAY_LOGO_TIME) {}
}

// ----------------------------------------------------------------------------

static void __fastcall__ gameMenuScreen() {
  AG_WAIT_LCD();

  /*
  set background image palette to show 'red' i.e. the 2nd (index 1) colour
  from the palette above in this 1-bit image; then draw the sprite it's linked to
  */
  agSprBackground.penpal[0] = 0x01;
  tgi_sprite(&agSprBackground);

  // draw some text after setting it's colour
  agSetFontColor(FONT_COLOUR_MENU);
  agDrawTextCenter(1, "GAME OPTIONS");
  agDrawTextCenter(AG_LCD_MAX_Y - AG_FONT_LINE_HEIGHT, "ANY KEY TO START");

  // update the LCD
  tgi_updatedisplay();

  // wait for any input before continuing
  AG_JOY_WAIT_INPUT();
}

// ----------------------------------------------------------------------------

static void __fastcall__ storyScreen() {
  AG_WAIT_LCD();

  /*
  set background image palette to show 'dark yellow' i.e. the 10th (index A) colour
  from the palette above in this 1-bit image; then draw the sprite it's linked to
  */
  agSprBackground.penpal[0] = 0x0A;
  tgi_sprite(&agSprBackground);

  // draw some text after setting it's colour
  agSetFontColor(FONT_COLOUR_STORY);
  agDrawTextCenter(10, "YOUR MISSION, SHOULD YOU");
  agDrawTextCenter(10 + AG_FONT_LINE_HEIGHT, "CHOOSE TO ACCEPT IT...");
  agDrawTextCenter(AG_LCD_MAX_Y - AG_FONT_LINE_HEIGHT, "ANY KEY TO START");

  // update the LCD
  tgi_updatedisplay();

  // wait for any input before continuing
  AG_JOY_WAIT_INPUT();
}

// ----------------------------------------------------------------------------

static void __fastcall__ resetGame() {
  playerX = AG_LCD_MAX_X / 2;
}

// ----------------------------------------------------------------------------

static void __fastcall__ updateAndDrawGame() {
  // process input - no out of bounds checks are done
  switch (SUZY.joystick) {
    case AG_JOY_RIGHT:
      playerX++;
      break;

    case AG_JOY_LEFT:
      playerX--;
      break;
  }

  AG_WAIT_LCD();

  /*
  set background image palette to show 'black' i.e. the 0th (index 0) colour
  from the palette above in this 1-bit image; then draw the sprite it's linked to
  */
  agSprBackground.penpal[0] = 0x00;
  tgi_sprite(&agSprBackground);

  // update game state
  sprPlayer.hpos = playerX;

  // draw the player sprite
  tgi_sprite(&sprPlayer);

  // update the LCD
  tgi_updatedisplay();
}

// ----------------------------------------------------------------------------
